package cz.mendelu.dnaAnalyser.analyse.g4hunter;

import cz.mendelu.dnaAnalyser.sequence.NucleicBuffer;
import cz.mendelu.dnaAnalyser.sequence.nucleotide.Nucleotide;
import org.springframework.stereotype.Service;

@Service
public class G4HunterAnalyser {
    
    private static final int G_SCORE = 1;
    private static final int C_SCORE = -1;
    private static final int OTHER_SCORE = 0;
    private static final int MAX_G_SCORE = 4;
    private static final int MIN_C_SCORE = -4;



    public int[] baseScore(NucleicBuffer nucleicBuffer) {
        int length = nucleicBuffer.getLength();
        int pos = 0;
        int[] baseScore = new int[length];
        Nucleotide nucleotide;
        
        while(pos < length) {
            nucleotide = nucleicBuffer.get(pos);
            switch (nucleotide) {
                case G:
                    updateScore(baseScore, pos, 1, G_SCORE);
                    if (pos + 1 < length && nucleicBuffer.get(pos + 1) == Nucleotide.G) {
                        updateScore(baseScore, pos, 2, G_SCORE);
                        if (pos + 2 < length && nucleicBuffer.get(pos + 2) == Nucleotide.G) {
                            updateScore(baseScore, pos, 3, G_SCORE);
                            if (pos + 3 < length && nucleicBuffer.get(pos + 3) == Nucleotide.G) {
                                updateScore(baseScore, pos, 4, G_SCORE);
                                pos++;
                            }
                            pos++;
                        }
                        pos++;
                    }
                    pos++;
                    while (pos < length && nucleicBuffer.get(pos) == Nucleotide.G) {
                        baseScore[pos] = MAX_G_SCORE;
                        pos++;
                    }
                    break;
                case C:
                    updateScore(baseScore, pos, 1, C_SCORE);
                    if (pos + 1 < length && nucleicBuffer.get(pos + 1) == Nucleotide.C) {
                        updateScore(baseScore, pos, 2, C_SCORE);
                        if (pos + 2 < length && nucleicBuffer.get(pos + 2) == Nucleotide.C) {
                            updateScore(baseScore, pos, 3, C_SCORE);
                            if (pos + 3 < length && nucleicBuffer.get(pos + 3) == Nucleotide.C) {
                                updateScore(baseScore, pos, 4, C_SCORE);
                                pos++;
                            }
                            pos++;
                        }
                        pos++;
                    }
                    pos++;
                    while (pos < length && nucleicBuffer.get(pos) == Nucleotide.C) {
                        baseScore[pos] = MIN_C_SCORE;
                        pos++;
                    }
                    break;
                default:
                    baseScore[pos++] = OTHER_SCORE;
                }
            }
        return baseScore;
    }

    private void updateScore(int[] score, int pos, int interact, int scoreValue) {
        for (int index = pos; index < pos + interact; index++) {
            score[index] = scoreValue * interact;
        }
    }

    public float[] windowsScore(int[] score, int window) {
        integrateScore(score);
        int length = score.length - window;
        float[] windowsScore = new float[length + 1];

        float sum = score[window - 1];
        float mean;
        windowsScore[0] = sum / window;
        for (int index = 0; index < length; index++) {
            sum = score[index + window] - score[index];
            mean = sum / window;
            windowsScore[index + 1] = mean;
        }
        return windowsScore;
    }

    public float score(int[] score) {
        return windowsScore(score, score.length)[0];
    }

    public float score(NucleicBuffer nucleicBuffer) {
        int[] baseScore = baseScore(nucleicBuffer);
        float result = score(baseScore);
        return result;
    }

    private void integrateScore(int[] score) {
        int sum = 0;
        for (int i = 0; i < score.length; i++) {
            score[i] = sum += score[i];
        }
    }



}
