package cz.mendelu.dnaAnalyser.sequence

import cz.mendelu.dnaAnalyser.sequence.nucleotide.Nucleotide
import spock.lang.Specification
import spock.lang.Unroll

import java.nio.charset.StandardCharsets

class NucleotideTest extends Specification {

    @Unroll
    def "test get(#b) -> #r"() {
        expect:
        Nucleotide.get(b.bytes[0]) == r

        where:
        b   || r
        'a' || Nucleotide.A
        'A' || Nucleotide.A
        'X' || Nucleotide.NONE

    }

    def "test utf-8 chars"() {
        expect:
        Nucleotide.get("ř".getBytes(StandardCharsets.UTF_8)[0]) == Nucleotide.NONE
        Nucleotide.get("ř".getBytes(StandardCharsets.UTF_8)[1]) == Nucleotide.NONE

        Nucleotide.getFromInternalFormat("ř".getBytes(StandardCharsets.UTF_8)[0]) == Nucleotide.NONE
        Nucleotide.getFromInternalFormat("ř".getBytes(StandardCharsets.UTF_8)[1]) == Nucleotide.NONE
    }

    @Unroll
    def "test get from internal format(#b) -> #r"() {
        expect:
        Nucleotide.getFromInternalFormat(b.bytes[0]) == r

        where:
        b   || r
        'A' || Nucleotide.A
        'X' || Nucleotide.NONE
        'C' || Nucleotide.C

    }

    @Unroll
    def "test #n isSupplement(#s, #t) -> #r"() {
        expect:
        n.isSupplement(s, t) == r

        where:
        n            | s            | t                || r
        Nucleotide.A | Nucleotide.T | SequenceType.DNA || true
        Nucleotide.T | Nucleotide.A | SequenceType.DNA || true
        Nucleotide.A | Nucleotide.U | SequenceType.RNA || true
        Nucleotide.U | Nucleotide.A | SequenceType.RNA || true
        Nucleotide.A | Nucleotide.T | SequenceType.RNA || false
        Nucleotide.T | Nucleotide.A | SequenceType.RNA || false
        Nucleotide.A | Nucleotide.U | SequenceType.DNA || false
        Nucleotide.U | Nucleotide.A | SequenceType.DNA || false
        Nucleotide.C | Nucleotide.G | SequenceType.DNA || true
        Nucleotide.G | Nucleotide.C | SequenceType.DNA || true
        Nucleotide.C | Nucleotide.G | SequenceType.RNA || true
        Nucleotide.G | Nucleotide.C | SequenceType.DNA || true
        Nucleotide.N | Nucleotide.A | SequenceType.DNA || false
        Nucleotide.B | Nucleotide.B | SequenceType.DNA || false
        Nucleotide.D | Nucleotide.R | SequenceType.DNA || false
    }

    @Unroll
    def "Testing #b is internal = #r"() {
        expect:
        Nucleotide.isNuclidInInernal(b) == r

        where:
        b                     || r
        Nucleotide.A.toByte() || true
        Nucleotide.C.toByte() || true
        Nucleotide.T.toByte() || true
        Nucleotide.G.toByte() || true
        Nucleotide.N.toByte() || true
        'a'.getBytes()[0]     || false
        'c'.getBytes()[0]     || false
        't'.getBytes()[0]     || false
        (byte)0               || false
        (byte)10              || false
        (byte)100             || false

    }
}
