package cz.mendelu.dnaAnalyser.sequence

import cz.mendelu.dnaAnalyser.sequence.inport.ImportSequenceService
import spock.lang.Specification

class SequenceServiceTest extends Specification {

    SequenceService sequenceService = new SequenceService()
    ImportSequenceService importSequenceService
    ModelSequenceService modelSequenceService


    UUID uuid
    byte[] buffer = 'ACGT'.bytes
    Sequence sequence = new Sequence(id: uuid)

    def setup() {
        sequenceService.importSequenceService = importSequenceService = Mock(ImportSequenceService)
        sequenceService.modelSequenceService = modelSequenceService = Mock(ModelSequenceService)

        uuid = UUID.randomUUID()
    }


    def "test findAll"() {
        when:
        sequenceService.findAll(null, null)

        then:
        1 * modelSequenceService.findAll(_, _)
    }

    def "test findOne"() {
        when:
        sequenceService.findOne(uuid)

        then:
        1 * modelSequenceService.findOne(uuid)
    }

    def "test getNcbi"() {
        when:
        sequenceService.getNcbi(uuid)

        then:
        1 * modelSequenceService.getNcbi(uuid)
    }

    def "test getName"() {
        when:
        sequenceService.getName(uuid)

        then:
        1 * modelSequenceService.getName(uuid)
    }

    def "test delete"() {
        when:
        sequenceService.delete(uuid)

        then:
        1 * modelSequenceService.delete(uuid)
    }

    def "test importPlainFormat"() {
        when:
        sequenceService.importPlainFormatFromText(sequence, buffer)

        then:
        1 * importSequenceService.importPlainFormatFromText(sequence, buffer)
    }

    def "test importFASTAFormat"() {
        when:
        sequenceService.importFASTAFormatFromText(sequence, buffer)

        then:
        1 * importSequenceService.importFASTAFormatFromText(sequence, buffer)
    }

    def "test importNCBI"() {
        when:
        sequenceService.importNCBI(sequence, 'ncbi')

        then:
        1 * importSequenceService.importNCBI(sequence, 'ncbi')
    }

    def "test modify tags"() {
        setup:
        def tags = ['test tags'] as Set<String>

        when:
        sequenceService.modifyTags(uuid, tags)

        then:
        1 * modelSequenceService.findOne(uuid) >> sequence
        1 * modelSequenceService.save(sequence)

        and:
        sequence.tags == tags
    }


}
