package cz.mendelu.dnaAnalyser.user

import cz.mendelu.dnaAnalyser.email.model.Email
import cz.mendelu.dnaAnalyser.email.service.EmailService
import cz.mendelu.dnaAnalyser.sequence.SequenceRepository
import spock.lang.Specification
import spock.lang.Subject

class UserServiceTest extends Specification {
    @Subject UserService userService

    def repository = Mock(UserRepository)
    def emailService = Mock(EmailService)
    def sequenceRepository

    def setup() {
        userService = new UserService()
        userService.userRepository = repository
        userService.emailService = emailService
        userService.sequenceRepository = sequenceRepository = Mock(SequenceRepository)
    }

    def "Registration new user."() {
        when:
        //def now = new Date().format('yyyy-MM-dd HH:mm')
        def user = userService.registrationUser('test@test.mendelu.cz', 'test')

        then:
        user != null
        1 * repository.save(_ as User) >> { User toSave -> toSave }

        and:
        user.id != null
        user.username == 'test@test.mendelu.cz'
        user.password != 'test'
        user.authorities == EnumSet.of(Authority.ROLE_USER)
        user.enabled == true
        //user.createdDate.format('yyyy-MM-dd HH:mm') == now
        //user.lastPasswordResetDate.format('yyyy-MM-dd HH:mm') == now

    }

    def "test changePassword"() {
        // FIXME domiplementuje mě.
    }

    def "test resetPassword"() {
        given:
        //def now = new Date().format('yyyy-MM-dd HH:mm')
        def user = new User(username: 'user@mendelu.cz')

        when:
        userService.resetPassword('user@mendelu.cz')

        then:
        1 * repository.save(_ as User) >> {User toSave -> user = toSave }
        1 * repository.findByUsername('user@mendelu.cz') >> Optional.of(user)
        1 * emailService.sendEmail(new Email('user@mendelu.cz'))
        user.password != null
        //user.lastPasswordResetDate.format('yyyy-MM-dd HH:mm') == now // FIXME Toto může eventáně selhat.
    }

    def "Check correct user password."() {
        given:
        def user = new User(
                password: userService.encoderPassword('user'))

        when:
        def result = userService.checkPassword(user, 'user')

        then:
        result == true

    }

    def "Check incorrect user password."() {
        given:
        def user = new User(
                password: userService.encoderPassword('user'))

        when:
        def result = userService.checkPassword(user, 'wrong')

        then:
        result == false
    }

    def "Test password encoder."() {
        when:
        def encoderPassword = userService.encoderPassword('user')

        then:
        encoderPassword != 'user'
    }

    def "Test delete user"() {
        given:
        def user = new User(
                id: UUID.randomUUID(),
                password: userService.encoderPassword('user'),
                enabled: true)

        when:
        userService.delete(user.id)

        then:
        1 * repository.findById(user.id) >> Optional.of(user)
        1 * repository.save(_ as User) >> {User toSave -> user = toSave }
        user.enabled == false
    }
}
